/*
 * Header file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
#ifndef __PGUIMAN__
  #define __PGUIMAN__
  
// Header files for working with postgresql
#include <libpq-fe.h>

// Basic header files
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

// Header files for GUI - GTK2 
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

// Local header files
#include "functions.h"

// Define pixmap directory
#define PIXMAP_DIRECTORY PACKAGE_DATA_DIR"/pguiman/pixmaps"

// Define glade file directory
#define GLADE_DIRECTORY PACKAGE_DATA_DIR"/pguiman/glade"

// loaded glade xml resource file
GladeXML *xml;

/*
 * Structure for inforamations about retrieved data
 */
typedef struct retrived_data{
	gint rows;
	gint cols;
	gint limit;
	gint offset;
	gchar * condition;
} T_Data;

/*
 * Structure for notebook page
 */
typedef struct page{
	gboolean modified;
	GtkWidget * page;
	gpointer object;
	gboolean (*close_function)(GtkButton * button, gpointer user_data);
	gchar * path;
} T_Page;

/*
 * Structure for inforamations about main notebook
 */
typedef struct notebook{
	gint n_pages;
	T_Page ** page;
} T_Notebook;

/*
 * Enumeration of column types
 */
enum{
	COL_ICON,
	COL_TEXT,
	NUM_COLS
};

/*
 * Structure for function parameters
 */
typedef struct param_struct{
	gchar * name;
	gchar * type;
	gchar * scope;
} T_Parameter;

/*
 * Structure with dependecy
 */
typedef struct dep{
	int OID;
	gchar * type;
} T_Depend;

/*
 * Structure for data types
 */
typedef struct dtypes_struct{
	int n_data_types;
	gchar ** name;
	gboolean * varlen;
} T_Data_types;

/*
 * Structure for roles
 */
typedef struct roles_struct{
	int n_roles;
	gchar ** name;
} T_Roles;

/*
 * Structure for tablespaces
 */
typedef struct tablespaces_struct{
	int n_tablespaces;
	gchar ** name;
} T_Tablespaces;

/*
 * Structure for dependecies
 */
typedef struct deps_struct{
	int n_deps;
	T_Depend * dep;
} T_Dependencies;

/*
 * Structure for columns
 */
typedef struct column_struct{
	int column_number;
	gboolean resizeable;
	// Common
	gchar * name;
	gchar * comment;
	// Field
	gchar * type;
	gchar * size;
	gchar * precision;
	gboolean pkey;
	gboolean unique;
	gboolean not_null;
	gchar * default_value;
	gint dimensions;
	gboolean inherited;
} T_Column;

/*
 * Structure with reference between columns
 */
typedef struct reference_struct{
	gchar * name;
	gint n_columns;                     // columns
	T_Column ** column;
	gchar * ref_schema;
	gchar * ref_table;
	gint n_ref_columns;               // ref_columns
	T_Column ** ref_column;
} T_Reference;

/*
 * Structure for foreign key
 */
typedef struct fkey_struct{
	// Common
  gchar * name;
  gchar * comment;
  int OID;
  // Foreign Key
	gint n_columns;                     // columns
	T_Column ** column;
	gchar * ref_schema;
	gchar * ref_table;
	gint n_ref_columns;               // ref_columns
	T_Column ** ref_column;
	gchar * on_update_rule;
	gchar * on_delete_rule;
	gchar * match_type;
	gboolean deferrable;
	gchar * check_time;
} T_FKey;

/*
 * Structure for index
 */
typedef struct index_struct{
	// Common
  gchar * name;
  gchar * comment;
  int OID;
  // Index
  gint n_columns;                     // columns
	T_Column ** column;
  gboolean pkey_constraint;
  gboolean unique_constraint;
  gboolean unique_index;
  gboolean clustered;
  gchar * predicate;
  gchar * tablespace;
  gchar * access_method;      
} T_Index;

/*
 * Structure for checks
 */
typedef struct check_struct{
	// Common
  gchar * name;
  gchar * comment;
  int OID;
  // Index
  gchar * condition;
} T_Check;

/*
 * Structure for trigger
 */
typedef struct trigger_struct{
	gchar * table_name;
	// Common
  gchar * name;
  gchar * comment;
  int OID;
	// Trigger
	gboolean enabled;
	gchar * type;
	gchar * for_each;
	gchar * events;
	gchar * function;
	gchar * arguments;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
} T_Trigger;

/*
 * Structure for rule
 */
typedef struct rules_struct{
	gchar * parent_name;
	// Common
  gchar * name;
  gchar * comment;
  int OID;
	// Rule
	gchar * event;
	gboolean instead;
	gchar * condition;
	// Definition
	gchar * definition;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
} T_Rule;

/*
 * Structure for foreign table
 */
typedef struct table_struct{
	T_Data data;
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
  // Table
  gboolean WithOIDs;
  gchar * inherited_from;
  gchar * tablespace;  
 	// Objects
	gint n_columns;                     // columns
	T_Column ** column;
	gint n_indexes;                      // indexes
	T_Index ** index;
	gint n_fkeys;                         // foreign keys
	T_FKey ** fkey;
	gint n_checks;                      // checks
	T_Check ** check;
	gint n_triggers;                      // triggers
	T_Trigger ** trigger;
	gint n_rules;                          // rules
	T_Rule ** rule;
	gint n_references;                // references
	T_Reference ** reference;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;
} T_Table;

/*
 * Structure for view
 */
typedef struct view_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
	// Body
	gchar * body;
 	// Objects
	gint n_columns;                     // columns
	T_Column ** column;
	gint n_rules;                          // rules
	T_Rule ** rule;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;	
} T_View;

/*
 * Structure for function
 */
typedef struct function_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
	// Function
	gchar * parameters;
	gboolean return_set;
	gchar * return_type;
	gchar * language;
	gboolean strict;
	gchar * execution_privilegies;
	gchar * stability;
	// Definition
	gchar * definition;
	// Parameters
	T_Parameter ** parameter;	// Parameters
	gint n_parameters;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;	
} T_Function;

/*
 * Structure for domain
 */
typedef struct domain_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
	// Domain
	gchar * data_type;
	gboolean not_null;
	gchar * default_value;
	// Objects
	gint n_checks;                      // checks
	T_Check ** check;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
} T_Domain;

/*
 * Structure for aggregate
 */
typedef struct aggregate_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
	// Aggregate
	gchar * base_type;
	gchar * state_type;
	gchar * state_function;
	gchar * final_type;
	gchar * final_function;
	gchar * initial_condition;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
} T_Aggregate;

/*
 * Structure for sequence
 */
typedef struct sequence_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
	// Sequence
	gchar * value;
	gchar * increment;
	gchar * max_value;
	gchar * min_value;
	gchar * cache;
	gboolean cycle;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;	
} T_Sequence;

/*
 * Structure for type
 */
typedef struct type_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
  // Type
  gchar * input_function;
  gchar * output_function;  
  gchar * receive_function;  
  gchar * send_function;
  gchar * analyze_function;  
  gchar * internal_length;
  gchar * Default;  
  gchar * element;
  gchar * delimiter;
  gchar * alignment;
  gboolean passed_by_value;
  gchar * storage;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
} T_Type;

/*
 * Structure for operator
 */
typedef struct operator_struct{
	// Common
  gchar * name;
  gchar * owner;
  gchar * comment;
  int OID;
  // Operator
  gchar * kind;
  gchar * function;  
  gchar * left_operand_type;  
  gchar * right_operand_type;
  gchar * commutator;  
  gchar * negator;
  gchar * restrict_function;  
  gchar * join_function;
  gchar * left_sort_operator;
  gchar * right_sort_operator;
  gchar * less_than_operator;
  gchar * grater_than_operator;
  gboolean supports_hash_joins;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
} T_Operator;

/*
 * Structure for schema
 */
typedef struct schema_struct{
	// Common
	gchar * name;
	gchar * owner;
	gchar * comment;
	int OID;
	// Objects
	gint n_tables;                        // tables
	T_Table ** table;
	gint n_views;                        // views
	T_View ** view;
	gint n_functions;                   // functions
	T_Function ** function;
	gint n_domains;                    // domains
	T_Domain ** domain;
	gint n_aggregates;                // aggregates
	T_Aggregate ** aggregate;
	gint n_sequences;                 // sequences
	T_Sequence ** sequence;
	gint n_types;                          // types
	T_Type ** type;
	gint n_operators;                   // operators
	T_Operator ** operator;
	gint n_triggers;                      // triggers
	T_Trigger ** trigger;
	gint n_rules;                          // rules
	T_Rule ** rule;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;
} T_Schema;

/*
 * Structure for language
 */
typedef struct language_struct{
	// Common
	gchar * name;
	gchar * comment;
	int OID;
	// Language
	gboolean trusted;
	gchar * handler_func;
	gchar * validator_func;
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;
} T_Language;

/*
 * Structure for cast
 */
typedef struct cast_struct{
	// Common
	gchar * name;
	gchar * comment;
	int OID;
	// Cast
	gchar * source_type;
	gchar * target_type;
	gchar * function;
	gchar * context;
} T_Cast;

/*
 * Structure for database
 */
typedef struct db_struct{
	gchar *template;
	// Common
	gchar * owner;
	gchar * comment;
	int OID;
	// Database
	int connection_limit;
	gchar * encoding;
	gboolean is_template;
	gchar * tablespace;
	int lastOID;	
	// Objects
	T_Schema ** schema;			// Schemas
	gint n_schemas;
	T_Language ** language; // Languages
	gint n_languages;
	T_Cast ** cast;									// Casts
	gint n_casts;
	// Permissions
	gchar * acl;	
} T_Database;

/*
 * Structure for role
 */
typedef struct role_struct{
	// Common
	gchar * name;
} T_Role;

/*
 * Structure for tablespace
 */
typedef struct tspace_struct{
	// Common
	gchar * name;
	gchar * owner;
	int OID;	
	// Dependencies
	T_Dependencies dep_on;
	T_Dependencies ref_by;
	// Permissions
	gchar * acl;
} T_Tablespace;

/*
 * Structure for database profile
 */
typedef struct profile_struct{
	gboolean modified;
	gchar * alias;
	PGconn * conn;
	gboolean connected;
	gchar * dbname;
	T_Database database;
} T_Profile;

/*
 * Structure for server inforamtions
 */
typedef struct server_struct{
	gchar *name;
	gint n_profiles;
	T_Profile ** profile;
	gint n_roles;
	T_Role ** role;
	gint n_tablespaces;
	T_Tablespace ** tablespace;
	gchar * host;
	gchar * uname;
	gchar * passwd;
	guint port;
	guint SSL;
} T_Server;

/*
 * Structure for whole object tree
 */
typedef struct objectTree{
	T_Server ** server;
	gint n_servers;
} T_ObjectTree;

/*
 * Global variables
 */
T_ObjectTree o_tree;     // object tree
T_Notebook notebook;  // informations about main notebook

gboolean some_db_connected; // if some database is connected
T_Profile * current_profile;          // pointer to currently selected profile

#endif
